﻿Public Class ACoAR

    Public model As ModelACoAR
    Public dictionary As IDictionaryStrategy
    Public convergence As ConvergenceCriterion
    Public clustering As IClusteringStrategy
    Public mergsplit As IMergingSplittingStrategy
    Public classifier As Classifier
    Public recalculation As maximunChangesLimitRecalculation
    Public naming As INamingStrategy
    Public similarities As ISimilaritiesStrategy

    Public Sub setModel(ByVal m As ModelACoAR)
        model = m
    End Sub

    Public Sub setDictionaryStrategy(ByVal ds As IDictionaryStrategy)
        dictionary = ds
        dictionary.setModel(model)
    End Sub

    Public Sub setClusteringStrategy(ByVal cs As IClusteringStrategy)
        clustering = cs
        clustering.setModel(model)
    End Sub

    Public Sub setConvergenceCriterion(ByVal c As ConvergenceCriterion)
        convergence = c
        convergence.setModel(model)
    End Sub

    Public Sub setMergingSplittingStrategy(ByVal ms As IMergingSplittingStrategy)
        mergsplit = ms
        mergsplit.setModel(model)
    End Sub

    Public Sub setClassifier(ByVal cl As Classifier)
        classifier = cl
        classifier.setModel(model)
    End Sub

    Public Sub setNamingStrategy(ByVal ns As INamingStrategy)
        naming = ns
        naming.setModel(model)
    End Sub

    Public Sub setSimilaritiesStrategy(ByVal ss As ISimilaritiesStrategy)
        similarities = ss
        similarities.setModel(model)
    End Sub

    Public Sub setRecalculationConditionCriterion(ByVal rc As RecalculationConditionCriterion)
        recalculation = rc
    End Sub

    Public Sub ModelACoAR_Creation()

        dictionary.create()

        For Each r In model.R
            If convergence.hasConverged(r) Then
                r.setConverged()
            Else
                r.setPending()
            End If
        Next

        clustering.create()
        dictionary.createVcRepresentation()
        mergsplit.process()

        For Each r In model.getResources("converged")
            If classifier.isCorrectlyClassified(r) Then
                r.setClassified()
            Else
                model.Z.drop(r)
            End If
        Next

        dictionary.createVcRepresentation()
        naming.process()

        similarities.calcSimilaritiesRR()
        similarities.calcSimilaritiesCC()
        similarities.calcSimilaritiesCR()
        similarities.calcSimilaritiesCD()

    End Sub

    Public Sub ModelACoAR_Evolution(ByVal tg As Tagging)
        If tg.type = "create" Then
            ModelACoAR_Evolution_create(tg)
        Else
            ModelACoAR_Evolution_delete(tg)
        End If
    End Sub

    Private Sub ModelACoAR_Evolution_create(ByVal tg As Tagging)
        If dictionary.inDictionary(tg.t) Then
            dictionary.updateVSM(tg)

            'Se avisa a la clase de racalculo que ha habido un nuevo cambio. Cuando se llega a 100 cambios se recalcula
            recalculation.incrementChanges()

            If tg.r.state = "pending" And convergence.hasConverged(tg.r) Then
                tg.r.setConverged()
            End If

            If tg.r.state = "converged" Then
                classifier.classify(tg.r)
            End If
        End If

        If recalculation.check Then
            recalculate()
        End If


    End Sub

    Private Sub recalculate()
        dictionary.update()
        clustering.update()
        mergsplit.process()
        dictionary.createVcRepresentation()
        naming.process()
        similarities.calcSimilaritiesCC()
        similarities.calcSimilaritiesCD()
        similarities.calcSimilaritiesCR()
        similarities.calcSimilaritiesRR()
    End Sub

    Private Sub ModelACoAR_Evolution_delete(ByVal tg As Tagging)
        If dictionary.inDictionary(tg.t) Then
            dictionary.updateVSM(tg)
        End If
    End Sub
End Class
