﻿Public Class Folksonomy
    Private _folksonomyName As String
    Public Property folksonomyName() As String
        Get
            Return _folksonomyName
        End Get
        Set(ByVal value As String)
            _folksonomyName = value
        End Set
    End Property

    Private _tagList As List(Of Tag)
    Public Property tagList() As List(Of Tag)
        Get
            Return _tagList
        End Get
        Set(ByVal value As List(Of Tag))
            _tagList = value
        End Set
    End Property

    Private _resourceList As List(Of Resource)
    Public Property resourceList() As List(Of Resource)
        Get
            Return _resourceList
        End Get
        Set(ByVal value As List(Of Resource))
            _resourceList = value
        End Set
    End Property

    Private _annotationList As List(Of Annotation)
    Public Property annotationList() As List(Of Annotation)
        Get
            Return _annotationList
        End Get
        Set(ByVal value As List(Of Annotation))
            _annotationList = value
        End Set
    End Property

    Public Sub New(ByVal name As String)
        folksonomyName = name
        tagList = New List(Of Tag)
        resourceList = New List(Of Resource)
        annotationList = New List(Of Annotation)
    End Sub

    Public Sub add(ByVal t As Tag)
        tagList.Add(t)
    End Sub
    Public Sub add(ByVal r As Resource)
        resourceList.Add(r)
    End Sub
    Public Sub add(ByVal a As Annotation)
        annotationList.Add(a)
    End Sub

    Public Function getTag(ByVal id As Integer) As Tag
        For Each t As Tag In tagList
            If t.tagId = id Then
                Return t
                Exit Function
            End If
        Next
        Return Nothing
    End Function

    Public Function getResource(ByVal id As Integer) As Resource
        For Each r As Resource In resourceList
            If r.id = id Then
                Return r
                Exit Function
            End If
        Next
        Return Nothing
    End Function

    Public Function getAnnotations(ByVal t As Tag) As Integer
        Dim n As Integer = t.AnnotationsNumber
        Return n
    End Function
    Public Function getAnnotations(ByVal r As Resource) As Integer
        Dim n As Integer = r.AnnotationsNumber
        Return n
    End Function
    Public Function getAnnotations(ByVal t As Tag, ByVal r As Resource) As Integer
        Dim a As Annotation = searchForAnnotation(t, r)
        If a IsNot Nothing Then
            Return a.AnnotationsNumber
        Else
            Return -1
        End If

    End Function

    Public Function searchForAnnotation(ByVal t As Tag, ByVal r As Resource) As Annotation
        For Each a As Annotation In annotationList
            If a.tag.text = t.text And a.resource.id = r.id Then
                Return a
                Exit Function
            End If
        Next
        Return Nothing
    End Function

    Public Function tagAt(ByVal n As Integer)
        If n <= tagList.Count - 1 Then
            Return tagList.Item(n)
        Else
            Return Nothing
        End If
    End Function

    Public Function resourceAt(ByVal n As Integer)
        If n <= resourceList.Count - 1 Then
            Return resourceList.Item(n)
        Else
            Return Nothing
        End If
    End Function

End Class

Public Class Tag


    Private _tagId As Integer
    Public Property tagId() As Integer
        Get
            Return _tagId
        End Get
        Set(ByVal value As Integer)
            _tagId = value
        End Set
    End Property

    Private _text As String
    Public Property text() As String
        Get
            Return _text
        End Get
        Set(ByVal value As String)
            _text = value
        End Set
    End Property

    Private _AnnotationsNumber As Integer
    Public ReadOnly Property AnnotationsNumber() As Integer
        Get
            Return _AnnotationsNumber
        End Get
    End Property

    Public Sub New(ByVal t As String, ByVal id As Integer)
        text = t
        tagId = id
        _AnnotationsNumber = 0
    End Sub
    Public Sub New(ByVal t As String, ByVal id As Integer, ByVal n As Integer)
        text = t
        tagId = id
        _AnnotationsNumber = n
    End Sub

    Public Sub setAnnotationsNumber(ByVal n As Integer)
        _AnnotationsNumber = n
    End Sub

    Public Sub increaseAnnotations()
        _AnnotationsNumber += 1
    End Sub
    Public Sub increaseAnnotations(ByVal n As Integer)
        _AnnotationsNumber += n
    End Sub

    Public Sub decreaseAnnotations()
        _AnnotationsNumber -= 1
    End Sub

    Public Overrides Function toString() As String
        Return Me.text + "   " + Me.AnnotationsNumber.ToString
    End Function

End Class

Public Class Resource

    Private _id As String
    Public Property id() As String
        Get
            Return _id
        End Get
        Set(ByVal value As String)
            _id = value
        End Set
    End Property

    Private _state As String
    Public ReadOnly Property state() As String
        Get
            Return _state
        End Get
    End Property

    Private _AnnotationsNumber As Integer
    Public ReadOnly Property AnnotationsNumber() As Integer
        Get
            Return _AnnotationsNumber
        End Get
    End Property

    Public Sub New(ByVal t As String)
        id = t
        _state = "pending"
        _AnnotationsNumber = 0
    End Sub
    Public Sub New(ByVal t As String, ByVal n As Integer)
        id = t
        _state = "pending"
        setAnnotationsNumber(n)
    End Sub

    Public Sub setAnnotationsNumber(ByVal n As Integer)
        _AnnotationsNumber = n
    End Sub

    Public Sub increaseAnnotations()
        _AnnotationsNumber += 1
    End Sub
    Public Sub increaseAnnotations(ByVal n As Integer)
        _AnnotationsNumber += n
    End Sub

    Public Sub decreaseAnnotations()
        _AnnotationsNumber -= 1
    End Sub

    Public Sub setPending()
        _state = "pending"
    End Sub

    Public Sub setConverged()
        _state = "converged"
    End Sub

    Public Sub setClassified()
        _state = "classified"
    End Sub

    Public Overrides Function toString() As String
        Return "R" + Me.id + "   " + Me.AnnotationsNumber.ToString + "   " + Me.state
    End Function

End Class

Public Class Annotation

    Private _tag As Tag
    Public Property tag() As Tag
        Get
            Return _tag
        End Get
        Set(ByVal value As Tag)
            _tag = value
        End Set
    End Property

    Private _resource As Resource
    Public Property resource() As Resource
        Get
            Return _resource
        End Get
        Set(ByVal value As Resource)
            _resource = value
        End Set
    End Property

    Public Sub New(ByVal t As Tag, ByVal r As Resource, ByVal n As Integer)
        tag = t
        resource = r
        setAnnotationsNumber(n)
    End Sub

    Private _AnnotationsNumber As Integer
    Public ReadOnly Property AnnotationsNumber() As Integer
        Get
            Return _AnnotationsNumber
        End Get
    End Property

    Public Sub setAnnotationsNumber(ByVal n As Integer)
        _AnnotationsNumber = n
    End Sub

    Public Sub increaseAnnotations()
        _AnnotationsNumber += 1
    End Sub

    Public Sub decreaseAnnotations()
        _AnnotationsNumber -= 1
    End Sub

    Public Overrides Function ToString() As String
        Return Me.tag.text + "     R" + Me.resource.id + "   " + Me.AnnotationsNumber.ToString
    End Function



End Class

Public Class Tagging

    Private _r As Resource
    Public Property r() As Resource
        Get
            Return _r
        End Get
        Set(ByVal value As Resource)
            _r = value
        End Set
    End Property

    Private _t As Tag
    Public Property t() As Tag
        Get
            Return _t
        End Get
        Set(ByVal value As Tag)
            _t = value
        End Set
    End Property

    Private _type As String
    Public Property type() As String
        Get
            Return _type
        End Get
        Set(ByVal value As String)
            _type = value
        End Set
    End Property

    Public Sub New(ByVal _r As Resource, ByVal _t As Tag, ByVal _type As String)
        r = _r
        t = _t
        type = _type
    End Sub


End Class