﻿Public MustInherit Class IClusteringStrategy
    Protected model As ModelACoAR
    Public resources As List(Of Resource)
    Public VRcon As List(Of vector)

    Public clusters As vector
    Public centroids As List(Of vector)

    Protected measure As SimilarityMeasure

    Public Sub setModel(ByVal m As ModelACoAR)
        model = m
    End Sub

    Public Sub setSimilarityMeasure(ByVal m As SimilarityMeasure)
        measure = m
    End Sub

    Public MustOverride Sub create()

    Public MustOverride Sub update()

End Class

Public Class ClusteringDistribKMeans
    Inherits IClusteringStrategy

    Public k As Integer
    Public slices As Integer

    

    Public Sub New()
        clusters = New vector
        centroids = New List(Of vector)
        VRcon = New List(Of vector)
        k = 0
    End Sub

    Public Sub setK(ByVal value As Integer)
        k = value
    End Sub

    Public Overrides Sub create()
        resources = model.getResources("converged")
        'Buscamos los vectores de representación de los recursos "converged"
        Dim index As Integer
        Dim vrep As vector
        For Each rec As Resource In resources
            index = model.R.IndexOf(rec)
            vrep = model.VR(index)
            VRcon.Add(vrep)
        Next


        If k = 0 Then
            k = Math.Round(Math.Sqrt(resources.Count / 2))
        End If

        assignInitialPosition()

        Dim cambios As Integer = clustering()
        Dim MAX_ITER As Integer = 1000
        Dim iter As Integer = 0

        While cambios > 0 And iter <= MAX_ITER
            recalculateCentroids()
            cambios = clustering()
            iter += 1
        End While

        'MessageBox.Show("Numero de iteraciones: " + iter.ToString)

        For i As Integer = 1 To centroids.Count
            model.C.add(i)
        Next
        model.Z.CreateClasification(clusters)

    End Sub

    Public Sub assignInitialPosition()
        Dim myRand As New Random
        Randomize()
        For Each r As Resource In resources
            clusters.add(myRand.Next(1, k + 1))
        Next

        recalculateCentroids()
    End Sub

    Public Function clustering() As Integer
        Dim cambios As Integer = 0

        Dim kmean As New KMeansWorker
        Dim results As vector

        kmean.setModel(model)
        results = kmean.Work(centroids)

        For i As Integer = 0 To clusters.length - 1
            If clusters.getValue(i) <> results.getValue(i) Then
                cambios += 1
                clusters.setValue(results.getValue(i), i)
            End If
        Next

        Return cambios
    End Function



    Public Sub recalculateCentroids()
        'Creamos los k centroides
        centroids.Clear()
        For i As Integer = 1 To k
            Dim centroide As New vector
            centroide.redimVector(model.D.Count)
            centroids.Add(centroide)
        Next
        
        Dim vrep As vector
        Dim clusterID As Integer
        For i As Integer = 0 To VRcon.Count - 1
            clusterID = clusters.getValue(i)
            Dim centroide As vector = centroids.Item(clusterID - 1)
            vrep = VRcon.Item(i)
            vector.vector_sum(centroide, vrep)
            centroids.Item(clusterID - 1) = centroide
        Next

    End Sub

    Public Overrides Sub update()

        k = Math.Round(Math.Sqrt(resources.Count / 2))
        assignEvolutionPositions()

        Dim cambios As Integer = clustering()
        Dim MAX_ITER As Integer = 1000
        Dim iter As Integer = 0

        While cambios > 0 And iter <= MAX_ITER
            recalculateCentroids()
            cambios = clustering()
            iter += 1
        End While

    End Sub

    Public Sub assignEvolutionPositions()

        centroids.Clear()
        For i As Integer = 1 To k
            If i <= model.C.values.Length Then
                Dim centroide = model.VC.ElementAt(i - 1)
                centroids.Add(centroide)
            Else
                Dim centroide As New vector
                centroide.redimVector(model.D.Count)
                centroids.Add(centroide)
                Dim myRand As New Random
                Randomize()
                clusters.redimVector(clusters.values.Length + 1)
                clusters.setValue(myRand.Next(1, k + 1), clusters.values.Length - 1)
            End If
        Next

        recalculateCentroids()
    End Sub


    Private Class KMeansWorker

        Private model As ModelACoAR

        Public Sub setModel(ByVal m As ModelACoAR)
            model = m
        End Sub

        Public Function Work(ByVal centroids As List(Of vector)) As vector

            Dim resources As List(Of Resource) = model.getResources("converged")
            Dim VRCon As New List(Of vector)

            For Each rec As Resource In resources
                Dim index As Integer = model.R.IndexOf(rec)
                Dim vrep As vector = model.VR(index)
                VRcon.Add(vrep)
            Next

            Dim results As New vector
            'results.redimVector(VRCon.Count)

            Dim measure As New SimilarityMeasureCosine
            Dim max_sim As Double
            Dim sim As Double
            Dim c_max As Integer

            For Each vrep As vector In VRCon
                max_sim = -1
                c_max = 0
                For Each cent As vector In centroids
                    sim = measure.similarity(vrep.values, cent.values)

                    If sim >= max_sim Then
                        max_sim = sim
                        c_max = centroids.IndexOf(cent) + 1
                    End If
                Next
                results.add(c_max)
            Next

            Return results
        End Function

    End Class
End Class